#!/usr/bin/r -t
#
# Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
#
# This file is part of Rcpp.
#
# Rcpp is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Rcpp is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.

.setUp <- function(){
	suppressMessages( require( inline ) )
}

test.CharacterVector <- function(){
	funx <- cfunction(signature(), '
	CharacterVector x(10) ;
	for( int i=0; i<10; i++) x[i] = "foo" ;
	return x ;', 
		Rcpp=TRUE, verbose=FALSE, includes = "using namespace Rcpp;" )
	checkEquals( funx(), rep("foo",10L), msg = "CharacterVector" )
}

test.CharacterVector.STRSXP <- function(){
	funx <- cfunction(signature(vec = "character" ), '
	CharacterVector x(vec) ;
	std::string st = "" ; 
	for( int i=0; i<x.size(); i++) { 
		st += x[i] ;
	}
	return wrap( st ) ;', 
		Rcpp=TRUE, verbose=FALSE, includes = "using namespace Rcpp;" )
	checkEquals( funx(letters), paste(letters,collapse="" ), 
		msg = "CharacterVector( STRSXP) " )
}

test.CharacterVector.initializer.list <- function(){
	if( Rcpp:::capabilities()[["initializer lists"]] ){
		funx <- cfunction(signature(), '
		CharacterVector x = {"foo", "bar"} ;
		return x ;', 
			Rcpp=TRUE, verbose=FALSE, 
			includes = "using namespace Rcpp;",
			cxxargs = "-std=c++0x" )
		checkEquals( funx(), c("foo","bar"), msg = "CharacterVector( initializer list) " )
	}
}

test.CharacterVector.plusequals <- function(){
	funx <- cfunction(signature(), '
	CharacterVector x(2) ;
	x[0] = "foo" ; 
	x[1] = "bar" ;
	x[0] += "bar" ;
	x[1] += x[0] ;
	return x ;
	', Rcpp=TRUE, verbose=FALSE, includes = "using namespace Rcpp;" )
	checkEquals( funx(), c("foobar", "barfoobar"), 
		msg = "StringProxy::operator+=" )
}

test.CharacterVector.matrix.indexing <- function(){

	funx <- cfunction(signature(x = "character" ), '
		CharacterVector m(x) ;
		std::string trace  ;
		for( size_t i=0 ; i<4; i++){
			trace += m(i,i) ;
		}
		return wrap( trace ) ;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	x <- matrix( as.character(1:16), ncol = 4 )
	checkEquals( funx(x), paste(diag(x), collapse = ""), msg = "matrix indexing" )
	
	y <- as.vector( x )
	checkException( funx(y) , msg = "not a matrix" )
	
	funx <- cfunction(signature(x = "integer" ), '
		CharacterVector m(x) ;
		for( size_t i=0 ; i<4; i++){
			m(i,i) = "foo" ;
		}
		return m ;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( diag(funx(x)), rep("foo", 4) , 
		msg = "matrix indexing lhs" )
}

test.CharacterVector.assign <- function(){
	
	funx <- cfunction(signature(), '
		const char* x[] = { "foo", "bar", "bling", "boom" } ;
		CharacterVector y ;
		y.assign( x, x+4 ) ;
		return y;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), c("foo", "bar", "bling", "boom"), msg = "assign(char**, char**)" )
	
	
	funx <- cfunction(signature(), '
		std::vector<std::string> vec(4) ;
		vec[0] = "foo";
		vec[1] = "bar";
		vec[2] = "bling";
		vec[3] = "boom" ;
		CharacterVector y ;
		y.assign( vec.begin(), vec.end() ) ;
		return y;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), c("foo", "bar", "bling", "boom"), msg = "assign(char**, char**)" )
	
}

test.CharacterVector.range.constructors <- function(){

	funx <- cfunction(signature(), '
		const char* x[] = { "foo", "bar", "bling", "boom" } ;
		CharacterVector y( x, x+4 ) ;
		return y;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), c("foo", "bar", "bling", "boom"), msg = "assign(char**, char**)" )
	
	
	funx <- cfunction(signature(), '
		std::vector<std::string> vec(4) ;
		vec[0] = "foo";
		vec[1] = "bar";
		vec[2] = "bling";
		vec[3] = "boom" ;
		CharacterVector y( vec.begin(), vec.end() ) ;
		return y;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), c("foo", "bar", "bling", "boom"), msg = "assign(char**, char**)" )
}

test.CharacterVector.Dimension.constructor <- function(){

	funx <- cfunction(signature(), '
		return CharacterVector( Dimension( 5 ) ) ;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), 
		character(5) , 
		msg = "CharacterVector( Dimension(5))" )
	
	funx <- cfunction(signature(), '
		return CharacterVector( Dimension( 5, 5 ) ) ;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), 
		matrix( "", ncol = 5, nrow = 5) , 
		msg = "CharacterVector( Dimension(5,5))" )
	
	funx <- cfunction(signature(), '
		return CharacterVector( Dimension( 2, 3, 4) ) ;
	', Rcpp = TRUE, includes = "using namespace Rcpp;"  )
	checkEquals( funx(), 
		array( "", dim = c(2,3,4) ) , 
		msg = "CharacterVector( Dimension(2,3,4))" )
}


